@@ -1,81 +1,45 @@
 from django import forms
 from django.contrib.auth.forms import UserCreationForm, UserChangeForm
 from django.contrib.auth.models import User
-from django.core.urlresolvers import reverse
 from django.utils.translation import ugettext_lazy as _
 
 from selectable import forms as selectable
 
-from timepiece.forms import SearchForm
+from timepiece.utils.search import SearchForm
 from timepiece.crm.lookups import BusinessLookup, ProjectLookup, UserLookup,\
         QuickLookup
 from timepiece.crm.models import Attribute, Business, Project,\
         ProjectRelationship, UserProfile
 
 
-class BusinessForm(forms.ModelForm):
+class CreateEditBusinessForm(forms.ModelForm):
 
     class Meta:
         model = Business
         fields = ('name', 'short_name', 'email', 'description', 'notes',)
 
 
-class ProjectForm(forms.ModelForm):
-    class Meta:
-        model = Project
-        fields = (
-            'name',
-            'business',
-            'tracker_url',
-            'point_person',
-            'type',
-            'status',
-            'activity_group',
-            'description',
-        )
-
-    business = selectable.AutoCompleteSelectField(
-        BusinessLookup,
-        label='Business',
-        required=True
-    )
+class CreateEditProjectForm(forms.ModelForm):
+    business = selectable.AutoCompleteSelectField(BusinessLookup)
     business.widget.attrs['placeholder'] = 'Search'
 
-    def __init__(self, *args, **kwargs):
-        super(ProjectForm, self).__init__(*args, **kwargs)
-
-    def save(self):
-        instance = super(ProjectForm, self).save(commit=False)
-        instance.save()
-        return instance
-
-
-class ProjectRelationshipForm(forms.ModelForm):
     class Meta:
-        model = ProjectRelationship
-        fields = ('types',)
-
-    def __init__(self, *args, **kwargs):
-        super(ProjectRelationshipForm, self).__init__(*args, **kwargs)
-        self.fields['types'].widget = forms.CheckboxSelectMultiple(
-            choices=self.fields['types'].choices
-        )
-        self.fields['types'].help_text = ''
+        model = Project
+        fields = ('name', 'business', 'tracker_url', 'point_person', 'type',
+                'status', 'activity_group', 'description')
 
 
-class UserProfileForm(forms.ModelForm):
+class CreateUserForm(UserCreationForm):
 
     class Meta:
-        model = UserProfile
-        exclude = ('user', 'hours_per_week')
+        model = User
+        fields = ('username', 'first_name', 'last_name', 'email', 'is_active',
+                'is_staff', 'groups')
 
-
-class SelectProjectForm(forms.Form):
-    project = selectable.AutoCompleteSelectField(ProjectLookup, label='')
-    project.widget.attrs['placeholder'] = 'Add Project'
-
-    def save(self):
-        return self.cleaned_data['project']
+    def __init__(self, *args, **kwargs):
+        super(CreateUserForm, self).__init__(*args, **kwargs)
+        self.fields['groups'].widget = forms.CheckboxSelectMultiple()
+        self.fields['groups'].help_text = None
 
 
 class EditUserForm(UserChangeForm):
@@ -85,6 +49,11 @@ class EditUserForm(UserChangeForm):
         label=_(u'Repeat Password'),
         widget=forms.PasswordInput(render_value=False))
 
+    class Meta:
+        model = User
+        fields = ('username', 'first_name', 'last_name', 'email', 'is_active',
+                'is_staff', 'groups')
+
     def __init__(self, *args, **kwargs):
         super(EditUserForm, self).__init__(*args, **kwargs)
 
@@ -95,9 +64,6 @@ class EditUserForm(UserChangeForm):
         if 'password' in self.fields:
             del(self.fields['password'])
 
-    def clean_password(self):
-        return self.cleaned_data.get('password_one', None)
-
     def clean(self):
         super(EditUserForm, self).clean()
         password_one = self.cleaned_data.get('password_one', None)
@@ -106,6 +72,9 @@ class EditUserForm(UserChangeForm):
             raise forms.ValidationError(_('Passwords Must Match.'))
         return self.cleaned_data
 
+    def clean_password(self):
+        return self.cleaned_data.get('password_one', None)
+
     def save(self, *args, **kwargs):
         commit = kwargs.get('commit', True)
         kwargs['commit'] = False
@@ -118,24 +87,32 @@ class EditUserForm(UserChangeForm):
             self.save_m2m()
         return instance
 
+
+class EditProjectRelationshipForm(forms.ModelForm):
+
     class Meta:
-        model = User
-        fields = ('username', 'first_name', 'last_name', 'email', 'is_active',
-                'is_staff', 'groups')
-
-
-class CreateUserForm(UserCreationForm):
+        model = ProjectRelationship
+        fields = ('types',)
 
     def __init__(self, *args, **kwargs):
-        super(CreateUserForm, self).__init__(*args, **kwargs)
+        super(EditProjectRelationshipForm, self).__init__(*args, **kwargs)
+        self.fields['types'].widget = forms.CheckboxSelectMultiple(
+                choices=self.fields['types'].choices)
 
-        self.fields['groups'].widget = forms.CheckboxSelectMultiple()
-        self.fields['groups'].help_text = None
+
+class EditUserProfileForm(forms.ModelForm):
 
     class Meta:
-        model = User
-        fields = ('username', 'first_name', 'last_name', 'email', 'is_active',
-                'is_staff', 'groups')
+        model = UserProfile
+        exclude = ('user', 'hours_per_week')
+
+
+class SelectProjectForm(forms.Form):
+    project = selectable.AutoCompleteSelectField(ProjectLookup, label='')
+    project.widget.attrs['placeholder'] = 'Add Project'
+
+    def save(self):
+        return self.cleaned_data['project']
 
 
 class SelectUserForm(forms.Form):
@@ -168,66 +145,17 @@ class ProjectSearchForm(SearchForm):
                 in Attribute.statuses.all()])
         self.fields['status'].choices = PROJ_STATUS_CHOICES
 
-    def save(self):
-        search = self.cleaned_data.get('search', '')
-        status = self.cleaned_data.get('status', '')
-        return (search, status)
-
-
-class DeleteForm(forms.Form):
-    """
-    Returns True if the object was deleted
-    """
-    def __init__(self, *args, **kwargs):
-        self.instance = kwargs.pop('instance', None)
-        super(DeleteForm, self).__init__(*args, **kwargs)
-
-    def save(self):
-        if self.instance:
-            try:
-                self.instance.delete()
-            except AssertionError:
-                return False
-            else:
-                return True
-        return False
-
 
 class QuickSearchForm(forms.Form):
-    quick_search = selectable.AutoCompleteSelectField(
-        QuickLookup,
-        label='Quick Search',
-        required=False
-    )
+    quick_search = selectable.AutoCompleteSelectField(QuickLookup, required=False)
     quick_search.widget.attrs['placeholder'] = 'Search'
 
     def clean_quick_search(self):
         item = self.cleaned_data['quick_search']
-
-        if item is not None:
-            try:
-                item = item.split('-')
-                if len(item) == 1 or '' in item:
-                    raise ValueError
-                return item
-            except ValueError:
-                raise forms.ValidationError('%s' %
-                    'User, business, or project does not exist')
-        else:
-            raise forms.ValidationError('%s' %
-                'User, business, or project does not exist')
+        if not item:
+            msg = 'No user, business, or project matches your query.'
+            raise forms.ValidationError(msg)
+        return item
 
     def save(self):
-        type, pk = self.cleaned_data['quick_search']
-
-        if type == 'individual':
-            return reverse('view_user', args=(pk,))
-        elif type == 'business':
-            return reverse('view_business', args=(pk,))
-        elif type == 'project':
-            return reverse('view_project', args=(pk,))
-
-        raise forms.ValidationError('Must be a user, project, or business')
-
-
-
+        return self.cleaned_data['quick_search'].get_absolute_url()
